@@ -3,12 +3,11 @@ package Crypt::CBC;
 use strict;
 use Carp 'croak','carp';
 use Crypt::CBC::PBKDF;
+use Crypt::URandom ();
 use bytes;
-use vars qw($VERSION);
 no warnings 'uninitialized';
-$VERSION = '3.04';
+our $VERSION = '3.07';
 
-use constant RANDOM_DEVICE      => '/dev/urandom';
 use constant DEFAULT_PBKDF      => 'opensslv1';
 use constant DEFAULT_ITER       => 10_000;  # same as OpenSSL default
 
@@ -72,11 +71,11 @@ sub new {
 
     # HEADER consistency
     if ($header_mode eq 'salt') {
-	croak "Cannot use salt-based key generation if literal key is specified"
+	croak "Cannot use -header mode of 'salt' if a literal key is specified or key derivation function is none"
 	    if $literal_key;
     }
     elsif ($header_mode eq 'randomiv') {
-	croak "Cannot encrypt using a non-8 byte blocksize cipher when using randomiv header mode"
+	croak "Cannot use -header mode of 'randomiv' in conjunction with a cipher whose blocksize greater than 8"
 	    unless $bs == 8
     }
 
@@ -618,16 +617,15 @@ sub pbkdf_obj {
 }
 
 ############################# generating key, iv and salt ########################
-# hopefully a replacement for mess below
 sub set_key_and_iv {
     my $self = shift;
 
-    if (!$self->{literal_key}) {
+    if ($self->pbkdf eq 'none' || $self->{literal_key}) {
+	$self->{iv} = $self->_get_random_bytes($self->blocksize) if $self->{make_random_iv};
+    } else {
 	my ($key,$iv) = $self->pbkdf_obj->key_and_iv($self->{salt},$self->{passphrase});
 	$self->{key} = $key;
 	$self->{iv}  = $iv if $self->{make_random_iv}; 
-    } else {
-	$self->{iv} = $self->_get_random_bytes($self->blocksize) if $self->{make_random_iv};
     }
 
     length $self->{salt} == 8                  or croak "Salt must be exactly 8 bytes long";
@@ -662,7 +660,7 @@ sub _read_key_and_iv {
 	croak "Ciphertext does not begin with a valid header for 'randomiv' header mode" unless defined $self->{iv};
 	croak "randomiv header mode cannot be used securely when decrypting with a >8 byte block cipher.\n"
 	    unless $self->blocksize == 8;
-	(undef,$self->{key}) = $self->pbkdf_obj->key_and_iv(undef,$self->{passphrase});
+	($self->{key},undef) = $self->pbkdf_obj->key_and_iv(undef,$self->{passphrase});
 	substr($$input_stream,0,16) = ''; # truncate
     }
 
@@ -768,14 +766,7 @@ sub random_bytes {
 sub _get_random_bytes {
   my $self   = shift;
   my $length = shift;
-  my $result;
-
-  if (-r RANDOM_DEVICE && open(F,RANDOM_DEVICE)) {
-    read(F,$result,$length);
-    close F;
-  } else {
-    $result = pack("C*",map {rand(256)} 1..$length);
-  }
+  my $result = Crypt::URandom::urandom($length);
   # Clear taint and check length
   $result =~ /^(.+)$/s;
   length($1) == $length or croak "Invalid length while gathering $length random bytes";
@@ -929,6 +920,7 @@ Crypt::CBC - Encrypt Data with Cipher Block Chaining Mode
   $key    = Crypt::CBC->random_bytes(8);  # assuming a 8-byte block cipher
   $iv     = Crypt::CBC->random_bytes(8);
   $cipher = Crypt::CBC->new(-pbkdf       => 'none',
+                            -header      => 'none',
                             -key         => $key,
                             -iv          => $iv);
 
@@ -1262,7 +1254,7 @@ keysize. This is useful for several of the newer algorithms, including
 AES, ARIA, Blowfish, and CAMELLIA. If -keysize is not specified, then
 Crypt::CBC will use the value returned by the cipher's max_keylength()
 method. Note that versions of CBC::Crypt prior to 2.36 could also
-allow you to set the blocksie, but this was never supported by any
+allow you to set the blocksize, but this was never supported by any
 ciphers and has been removed.
 
 For compatibility with earlier versions of this module, you can
@@ -1430,10 +1422,9 @@ B<literal_key> is false.
 
 =head2 $data = random_bytes($numbytes)
 
-Return $numbytes worth of random data. On systems that support the
-"/dev/urandom" device file, this data will be read from the
-device. Otherwise, it will be generated by repeated calls to the Perl
-rand() function.
+Return $numbytes worth of random data, using L<Crypt::URandom>, which
+will read data from the system's source of random bytes, such as
+F</dev/urandom>.
 
 =head2 cipher(), pbkdf(), padding(), keysize(), blocksize(), chain_mode() 
 
@@ -1544,6 +1535,8 @@ Please report them.
 
 Lincoln Stein, lstein@cshl.org
 
+=head1 LICENSE
+
 This module is distributed under the ARTISTIC LICENSE v2 using the
 same terms as Perl itself.
 
